/*
* Copyright (c) 2003, University of Maryland 
* Copyright (c) 2003, Center for Satellite and Hybrid Communication Networks 
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the University of Maryland/Center for Satellite
*     and Hybrid Communication Networks nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

/* Authors: 
*     Jonathan Polley (jpolley@isr.umd.edu),
*     Dionysus Blazakis (dblaze@isr.umd.edu),
*     Jonathan McGee (mcgee@isr.umd.edu),
*     Daniel Rusk (ruskd@isr.umd.edu),
*     Karl Seamon (kks@isr.umd.edu),
*     Manish Karir (karir@isr.umd.edu)
*/

#include <stdarg.h>

#include "debug.h"

static decode_mode_t decode_mode=DECODE_NORMAL;
static FILE *decode_file=NULL;

static void log_header(atmega_chip *chip, char *label){
    printf("%03d %20lld %s", chip->id, chip->cycles, label);
}

void log_opcode(atmega_chip *chip, char *format, ...){
    va_list args;

    if(decode_mode == DECODE_NORMAL){
        log_header(chip, "opcode ");
    }
    fprintf(decode_file, "A0x%06x:\t", chip->pc * 2);
    va_start(args, format);
    vfprintf(decode_file, format, args);
    va_end(args);

    fflush(stdout);
}

void log_debug(atmega_chip *chip, int level, char *format, ...){
    if(verbosity >= level){
        va_list args;

        log_header(chip, "debug ");
        va_start(args, format);
        vprintf(format, args);
        va_end(args);

        fflush(stdout);
    }
}

void log_dump(atmega_chip *chip, char *format, ...){
    va_list args;

    log_header(chip, "data ");
    va_start(args, format);
    vprintf(format, args);
    va_end(args);

    fflush(stdout);
}

void log_error(atmega_chip *chip, char *format, ...){
    va_list args;

    log_header(chip, "error ");
    va_start(args, format);
    vprintf(format, args);
    va_end(args);

    fflush(stdout);
}

void ioman_device_event(device_obj *dev, char *format, ...){
    va_list args;

    log_header(dev->chip, "event ");
    printf("%s %s ", dev->type, dev->name);
    va_start(args, format);
    vprintf(format, args);
    va_end(args);

    fflush(stdout);
}

void ioman_device_debug(device_obj *dev, int level, char *format, ...){
    if(verbosity >= level){
        va_list args;

        log_header(dev->chip, "debug ");
        printf("%s %s ", dev->type, dev->name);
        va_start(args, format);
        vprintf(format, args);
        va_end(args);

        fflush(stdout);
    }
}

void set_decode_mode(decode_mode_t mode, char *file){
    decode_mode=mode;

    if(decode_file != stdout && decode_file != NULL){
        fclose(decode_file);
    }

    if(file != NULL){
        decode_file=fopen(file, "w");
        if(decode_file == NULL){
            printf("Error opening '%s' for writing.\n", file);
            decode_file=stdout;
        }
    }else{
        decode_file=stdout;
    }
}
