/*
* Copyright (c) 2003, University of Maryland 
* Copyright (c) 2003, Center for Satellite and Hybrid Communication Networks 
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the University of Maryland/Center for Satellite
*     and Hybrid Communication Networks nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

/* Authors: 
*     Jonathan Polley (jpolley@isr.umd.edu),
*     Dionysus Blazakis (dblaze@isr.umd.edu),
*     Jonathan McGee (mcgee@isr.umd.edu),
*     Daniel Rusk (ruskd@isr.umd.edu),
*     Karl Seamon (kks@isr.umd.edu),
*     Manish Karir (karir@isr.umd.edu)
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "callbacks.h"
#include "toolbar.h"
#include "interface.h"
#include "support.h"
#include "xatdb_elf.h"
#include "table.h"
#include "gtk_helper.h"
#include "chip_data.h"
#include "chip_data_struct.h"
#include "atemu.h"
#include "func_data.h"
#include "breakpoints.h"
#include "gutter.h"
#include "dialogs.h"

int verbose_level = 1;

double vpane_ratio, hpane_ratio;

GtkItemFactory *item_factory;
static GtkItemFactoryEntry treeview_break_popup_item =
        { "/Clear Breakpoint", NULL, break_popup_clear_break,
             0, "<StockItem>", XATDB_STOCK_BREAK_OFF };

static GtkItemFactoryEntry treeview_watch_popup_item =
        { "/Clear Watchpoint", NULL, watch_popup_clear_watch,
            0, "<StockItem>", XATDB_STOCK_WATCH_OFF };

GtkTextTagTable *tagtable;

GtkWidget *notebook_chips;
    
GtkWidget *button_step;
GtkWidget *button_go;
GtkWidget *button_cstep;
GtkWidget *button_stop;
GtkWidget *button_restart;
GtkWidget *button_set_break;
GtkWidget *button_set_watch;
GtkWidget *menu_debug_step;
GtkWidget *menu_debug_cstep;
GtkWidget *menu_debug_run;
GtkWidget *menu_debug_stop;
GtkWidget *menu_debug_restart;
GtkWidget *menu_debug_add_break_at;
GtkWidget *menu_debug_clear_breaks;
GtkWidget *menu_debug_set_watch;
GtkWidget *menu_debug_clear_watch;
GtkWidget *menu_debug_add_watch_at;
GtkWidget *menu_debug_clear_watches;
GtkWidget *menu_edit_copy;
GtkWidget *menu_edit_select_all;
GtkWidget *menu_edit_deselect_all;
GtkTooltips *tooltips;
GtkWidget *statusbar1;
GtkWidget *progressbar1;
GtkWidget *status_table;

GtkWidget *textview_console;
GtkTextBuffer *textbuff_console;

GtkWidget *window1;

static GtkCellRenderer *renderer;
static GtkCellRenderer *data_renderer;
static GtkCellRenderer *pix_renderer;

void create_chip_tab(GtkWidget *notebook_chips, int ntab){
    char tab_label[20];
    
    GtkWidget *scrolledwindow_break = gtk_scrolled_window_new (NULL, NULL);
    GtkWidget *scrolledwindow_stack = gtk_scrolled_window_new (NULL, NULL);
    GtkWidget *scrolledwindow_regs = gtk_scrolled_window_new (NULL, NULL);
    GtkWidget *scrolledwindow_locals = gtk_scrolled_window_new (NULL, NULL);
    GtkWidget *scrolledwindow_globals = gtk_scrolled_window_new (NULL, NULL);
    GtkWidget *scrolledwindow_events = gtk_scrolled_window_new (NULL, NULL);
    GtkWidget *scrolledwindow_debug = gtk_scrolled_window_new (NULL, NULL);
    GtkWidget *scrolledwindow_watch = gtk_scrolled_window_new (NULL, NULL);
    
    GtkWidget *notebook_data = gtk_notebook_new ();
    GtkWidget *notebook_debugger = gtk_notebook_new ();
    
    GtkWidget *label_regs = gtk_label_new (_("Registers"));
    GtkWidget *label_stack = gtk_label_new (_("Stack"));
    GtkWidget *label_locals = gtk_label_new (_("Locals"));
    GtkWidget *label_globals = gtk_label_new (_("Globals"));
    GtkWidget *label_events = gtk_label_new (_("Events"));
    GtkWidget *label_debug = gtk_label_new (_("Debug"));
    GtkWidget *label_breaks = gtk_label_new (_("Breakpoints"));
    GtkWidget *label_watch = gtk_label_new (_("Watchpoints"));
    GtkWidget *label_source = gtk_label_new(_("Source: "));
    GtkWidget *label_tab;
    
    GtkWidget *vbox1;
    GtkWidget *hbox1;
    
    GtkWidget *disassem_item;
    GtkWidget *frame;
    GtkTreeViewColumn *column;
    chip_data *data = chip_array + ntab; /* shortcut to the current chip's data */
    int x, y, hpos, vpos;
    
    data->id = ntab;
    
    /* build gui elements.  Takes a lot of code */
    data->vpaned = gtk_vpaned_new ();
    gtk_container_add (GTK_CONTAINER (notebook_chips), data->vpaned);
    vpane_ratio = 0.53;
    g_signal_connect(G_OBJECT(data->vpaned), "notify::position", G_CALLBACK(vpane_change_position), NULL);

    data->hpaned = gtk_hpaned_new ();
    gtk_paned_pack1 (GTK_PANED (data->vpaned), data->hpaned, FALSE, TRUE);
    hpane_ratio = 0.65;
    g_signal_connect(G_OBJECT(data->hpaned), "notify::position", G_CALLBACK(hpane_change_position), NULL);

    vbox1 = gtk_vbox_new(FALSE, 2);
    
    gtk_paned_pack1(GTK_PANED(data->hpaned), vbox1, FALSE, TRUE);

    hbox1 = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(hbox1), label_source, FALSE, FALSE, 0);

    data->cfile_select_menu = gtk_menu_new();
    
    data->cfile_select = gtk_option_menu_new();

    /* until we've added a source file, leave the dropdown insensitive */
    gtk_widget_set_sensitive( GTK_WIDGET(data->cfile_select), FALSE );

    gtk_box_pack_start(GTK_BOX(hbox1), data->cfile_select, TRUE, TRUE, 0);
    gtk_option_menu_set_menu(GTK_OPTION_MENU(data->cfile_select), data->cfile_select_menu);

    /* Add the Disassembly item to the list */
    disassem_item = gtk_menu_item_new_with_label( "Disassembly" );
    gtk_menu_shell_append( GTK_MENU_SHELL( data->cfile_select_menu ), disassem_item );
    gtk_option_menu_set_history(GTK_OPTION_MENU(data->cfile_select), 0);
    
    g_signal_connect( G_OBJECT( data->cfile_select ), "changed",
                G_CALLBACK( select_cfile ), data );

    /* make vbox for the textviews */
    frame = gtk_frame_new(NULL);
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
    data->vbox_source = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX (vbox1), frame, TRUE, TRUE, 0);
    gtk_container_add(GTK_CONTAINER(frame), data->vbox_source);

    /* Prepare the Disassembly textview */
    data->code_scroll = GTK_SCROLLED_WINDOW( gtk_scrolled_window_new (NULL, NULL) );
    data->textview_code = gtk_text_view_new ();
    gutter_init(GTK_TEXT_VIEW(data->textview_code), 2);
    
    gtk_text_view_set_editable( GTK_TEXT_VIEW( data->textview_code ), FALSE );
    //gtk_text_view_set_cursor_visible( GTK_TEXT_VIEW( data->textview_code ), FALSE );
    
    /* Set textview_code as the current textview */
    data->textview_source = data->textview_code;
    
    gtk_container_add (GTK_CONTAINER (data->code_scroll), data->textview_code);
    gtk_box_pack_start( GTK_BOX( data->vbox_source ), GTK_WIDGET( data->code_scroll ), TRUE, TRUE, 0 );
    
    g_signal_connect(G_OBJECT(data->textview_code), "populate-popup", G_CALLBACK(code_window_popup), NULL);
    g_signal_connect(G_OBJECT(data->textview_code), "button-press-event", G_CALLBACK(mark_to_click), NULL);
    g_signal_connect(G_OBJECT(data->textview_code), "event-after", G_CALLBACK(text_selected), NULL);
    
    gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook_data), TRUE);
    gtk_paned_pack2 (GTK_PANED (data->hpaned), notebook_data, TRUE, TRUE);
    
    gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( scrolledwindow_regs ), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS );
    gtk_container_add (GTK_CONTAINER (notebook_data), scrolledwindow_regs);

    data->treeview_regs = gtk_tree_view_new ();
    
    column = gtk_tree_view_column_new_with_attributes (_("Register"),
                                                   renderer,
                                                   "text", REGS_NAME,
                                                   NULL);
                                                   
    gtk_tree_view_append_column (GTK_TREE_VIEW (data->treeview_regs), column);

    column = gtk_tree_view_column_new_with_attributes (_("Value (Hex)"),
                                                   data_renderer,
                                                   "text", REGS_HEX,
                                                   NULL);
                                               
    gtk_tree_view_append_column (GTK_TREE_VIEW (data->treeview_regs), column);
    
    column = gtk_tree_view_column_new_with_attributes (_("Value (Dec)"),
                                                   data_renderer,
                                                   "text", REGS_DEC,
                                                   NULL);
                                                   
    gtk_tree_view_append_column (GTK_TREE_VIEW (data->treeview_regs), column);

    gtk_container_add (GTK_CONTAINER (scrolledwindow_regs), data->treeview_regs);

    gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_data), data->treeview_regs->parent, label_regs);
    gtk_label_set_justify (GTK_LABEL (label_regs), GTK_JUSTIFY_LEFT);

    gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( scrolledwindow_stack ), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS );
    gtk_container_add (GTK_CONTAINER (notebook_data), scrolledwindow_stack);

    data->treeview_stack = gtk_tree_view_new ();

    column = gtk_tree_view_column_new_with_attributes (_("Call Stack - Top"),
                                                   renderer,
                                                   "text", STACK_SFILE,
                                                   NULL);
                                                   
    gtk_tree_view_append_column (GTK_TREE_VIEW (data->treeview_stack),
        column);

    gtk_container_add (GTK_CONTAINER (scrolledwindow_stack), data->treeview_stack);

    gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_data), data->treeview_stack->parent, label_stack);
    gtk_label_set_justify (GTK_LABEL (label_stack), GTK_JUSTIFY_LEFT);

    gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( scrolledwindow_locals ), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS );
    gtk_container_add (GTK_CONTAINER (notebook_data), scrolledwindow_locals);

    data->treeview_locals = gtk_tree_view_new ();

    column = gtk_tree_view_column_new_with_attributes (_("Name"),
                                                   renderer,
                                                   "text", LOCALS_NAME,
                                                   NULL);
                                                   
    gtk_tree_view_append_column (GTK_TREE_VIEW (data->treeview_locals),
        column);

    column = gtk_tree_view_column_new_with_attributes (_("Value"),
                                                   data_renderer,
                                                   "text", LOCALS_DATA,
                                                   NULL);
                                                   
    gtk_tree_view_append_column (GTK_TREE_VIEW (data->treeview_locals),
        column);

    gtk_container_add (GTK_CONTAINER (scrolledwindow_locals), data->treeview_locals);

    g_signal_connect(G_OBJECT(data->treeview_locals), "button-press-event", G_CALLBACK(click_locals), NULL);

    gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_data), data->treeview_locals->parent, label_locals);
    gtk_label_set_justify (GTK_LABEL (label_locals), GTK_JUSTIFY_LEFT);

    gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( scrolledwindow_globals ), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS );
    gtk_container_add (GTK_CONTAINER (notebook_data), scrolledwindow_globals);

    data->treeview_globals = gtk_tree_view_new ();
    g_signal_connect( G_OBJECT( gtk_tree_view_get_selection( GTK_TREE_VIEW( data->treeview_globals ) ) ),
        "changed", G_CALLBACK( treeview_globals_selection_changed ), NULL );
    g_signal_connect( G_OBJECT( data->treeview_globals ), "button_release_event",
        G_CALLBACK( treeview_globals_selection_changed ), NULL );
    g_signal_connect( G_OBJECT( data->treeview_globals ), "focus-in-event",
        G_CALLBACK( treeview_globals_selection_changed ), NULL );
    g_signal_connect( G_OBJECT( data->treeview_globals ), "focus-out-event",
        G_CALLBACK( watch_focus_out ), NULL );

    column = gtk_tree_view_column_new_with_attributes (" ",
                                                   pix_renderer,
                                                   "stock-id", GLOBALS_ICON,
                                                   NULL);
    gtk_tree_view_column_set_min_width( column, 24 );
    gtk_tree_view_append_column (GTK_TREE_VIEW (data->treeview_globals),
        column);

    column = gtk_tree_view_column_new_with_attributes (_("Name"),
                                                   renderer,
                                                   "text", GLOBALS_NAME,
                                                   NULL);
                                                   
    gtk_tree_view_append_column (GTK_TREE_VIEW (data->treeview_globals),
        column);
    gtk_tree_view_set_expander_column( GTK_TREE_VIEW(data->treeview_globals), column );

    column = gtk_tree_view_column_new_with_attributes (_("Value"),
                                                   data_renderer,
                                                   "text", GLOBALS_DATA,
                                                   NULL);
                                                   
    gtk_tree_view_append_column (GTK_TREE_VIEW (data->treeview_globals),
        column);

    gtk_container_add (GTK_CONTAINER (scrolledwindow_globals), data->treeview_globals);

    gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_data), data->treeview_globals->parent, label_globals);
    gtk_label_set_justify (GTK_LABEL (label_globals), GTK_JUSTIFY_LEFT);

    g_signal_connect(G_OBJECT(data->treeview_globals), "button-press-event", G_CALLBACK(click_globals), NULL);

    gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook_debugger), TRUE);
    gtk_paned_pack2 (GTK_PANED (data->vpaned), notebook_debugger, TRUE, TRUE);

    gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( scrolledwindow_events ), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS );
    gtk_container_add (GTK_CONTAINER (notebook_debugger), scrolledwindow_events);

    data->textview_events = gtk_text_view_new ();
    gtk_text_view_set_editable( GTK_TEXT_VIEW( data->textview_events ), FALSE );
    gtk_text_view_set_cursor_visible( GTK_TEXT_VIEW( data->textview_events ), FALSE );
    gtk_container_add (GTK_CONTAINER (scrolledwindow_events), data->textview_events);
    
    g_signal_connect(G_OBJECT(data->textview_events), "event-after", G_CALLBACK(text_selected), NULL);
    
    gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_debugger), data->textview_events->parent, label_events);
    gtk_label_set_justify (GTK_LABEL (label_events), GTK_JUSTIFY_LEFT);

    gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( scrolledwindow_debug ), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS );
    gtk_container_add (GTK_CONTAINER (notebook_debugger), scrolledwindow_debug);

    data->textview_debug = gtk_text_view_new ();
    gtk_text_view_set_editable( GTK_TEXT_VIEW( data->textview_debug ), FALSE );
    gtk_text_view_set_cursor_visible( GTK_TEXT_VIEW( data->textview_debug ), FALSE );
    gtk_container_add (GTK_CONTAINER (scrolledwindow_debug), data->textview_debug);

    g_signal_connect(G_OBJECT(data->textview_debug), "event-after", G_CALLBACK(text_selected), NULL);

    gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_debugger), data->textview_debug->parent, label_debug);
    gtk_label_set_justify (GTK_LABEL (label_debug), GTK_JUSTIFY_LEFT);

    gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( scrolledwindow_break ), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS );
    gtk_container_add (GTK_CONTAINER (notebook_debugger), scrolledwindow_break);

    /*popup menu for treeview break*/
    item_factory = gtk_item_factory_new( 
        GTK_TYPE_MENU, "<treeviewbreakpopup>", NULL );
    gtk_item_factory_create_item( item_factory, 
        &treeview_break_popup_item, NULL, 1 );
    data->treeview_break_popup =  GTK_WIDGET(gtk_item_factory_get_widget( item_factory, 
                    "<treeviewbreakpopup>"));

    data->treeview_break = gtk_tree_view_new ();
    
    column = gtk_tree_view_column_new_with_attributes (_("Address"),
                                                   renderer,
                                                   "text", BREAK_ADDR_TEXT,
                                                   NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (data->treeview_break), column);

    column = gtk_tree_view_column_new_with_attributes (_("Source Line"),
                                                   renderer,
                                                   "text", BREAK_LINE,
                                                   NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (data->treeview_break), column);
    
    g_signal_connect(G_OBJECT(data->treeview_break), "button-press-event", G_CALLBACK(break_window_popup), NULL);
    gtk_container_add (GTK_CONTAINER (scrolledwindow_break), data->treeview_break);

    gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_debugger), data->treeview_break->parent, label_breaks);
    gtk_label_set_justify (GTK_LABEL (label_breaks), GTK_JUSTIFY_LEFT);

    gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( scrolledwindow_watch ), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS );
    gtk_container_add (GTK_CONTAINER (notebook_debugger), scrolledwindow_watch);

    /*popup menu for treeview break*/
    item_factory = gtk_item_factory_new( 
        GTK_TYPE_MENU, "<treeviewwatchpopup>", NULL );
    gtk_item_factory_create_item( item_factory, 
        &treeview_watch_popup_item, NULL, 1 );
    data->treeview_watch_popup =  GTK_WIDGET(gtk_item_factory_get_widget( item_factory, 
                    "<treeviewwatchpopup>"));
    
    data->treeview_watch = gtk_tree_view_new ();
    g_signal_connect( G_OBJECT( gtk_tree_view_get_selection( GTK_TREE_VIEW( data->treeview_watch ) ) ),
        "changed", G_CALLBACK( treeview_watch_selection_changed ), NULL );
    g_signal_connect( G_OBJECT( data->treeview_watch ), "button_release_event",
        G_CALLBACK( treeview_watch_selection_changed ), NULL );
    g_signal_connect( G_OBJECT( data->treeview_watch ), "focus-in-event",
        G_CALLBACK( treeview_watch_selection_changed ), NULL );
    g_signal_connect( G_OBJECT( data->treeview_watch ), "focus-out-event",
        G_CALLBACK( watch_focus_out ), NULL );
    
    column = gtk_tree_view_column_new_with_attributes (_("Address"),
                                                   renderer,
                                                   "text", WATCH_LOC,
                                                   NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (data->treeview_watch), column);
    
    column = gtk_tree_view_column_new_with_attributes (_("Name"),
                                                   renderer,
                                                   "text", WATCH_NAME,
                                                   NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (data->treeview_watch), column);
    
    g_signal_connect(G_OBJECT(data->treeview_watch), "button-press-event", G_CALLBACK(watch_window_popup), NULL);
    gtk_container_add (GTK_CONTAINER (scrolledwindow_watch), data->treeview_watch);

    gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_debugger), data->treeview_watch->parent, label_watch);
    gtk_label_set_justify (GTK_LABEL (label_watch), GTK_JUSTIFY_LEFT);

    snprintf(tab_label, sizeof(tab_label), "%s %d", _("Chip"), ntab);
    tab_label[sizeof(tab_label)-1] = '\0';
    label_tab = gtk_label_new (tab_label);
    
    gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_chips), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook_chips), ntab + 1), label_tab);
    gtk_label_set_justify (GTK_LABEL (label_tab), GTK_JUSTIFY_LEFT);
gtk_widget_hide( gtk_notebook_get_nth_page( GTK_NOTEBOOK( notebook_chips ), ntab + 1 ) );
    /* set text buffers and tree models */    
    gtk_text_view_set_buffer( GTK_TEXT_VIEW( data->textview_code ), data->code );
    gtk_tree_view_set_model( GTK_TREE_VIEW( data->treeview_regs ), GTK_TREE_MODEL( data->regs ) );
    gtk_tree_view_set_model( GTK_TREE_VIEW( data->treeview_stack ), GTK_TREE_MODEL( data->stack ) );
    gtk_tree_view_set_model( GTK_TREE_VIEW( data->treeview_locals ), GTK_TREE_MODEL( data->locals ) );
    gtk_tree_view_set_model( GTK_TREE_VIEW( data->treeview_globals ), GTK_TREE_MODEL( data->globals ) );
    gtk_text_view_set_buffer( GTK_TEXT_VIEW( data->textview_events ), data->events );
    gtk_text_view_set_buffer( GTK_TEXT_VIEW( data->textview_debug ), data->debug );
    gtk_tree_view_set_model( GTK_TREE_VIEW( data->treeview_break ), GTK_TREE_MODEL( data->breakp ) );
    gtk_tree_view_set_model( GTK_TREE_VIEW( data->treeview_watch ), GTK_TREE_MODEL( data->watchp ) );

    /* set initial pane positions */
    gtk_window_get_size(GTK_WINDOW(window1), &x, &y);
    hpos = (int)(hpane_ratio * ((double)x));
    vpos = (int)(vpane_ratio * ((double)y));

    gtk_paned_set_position(GTK_PANED (data->hpaned), hpos);
    gtk_paned_set_position(GTK_PANED (data->vpaned), vpos);
}

/* this assumes each buffer contains exactly ONE 
   line of output... */  
void process_buffer( char *buff )
{
    int chipid, left, num_chips;
    uint64_t cycles;
    char label[64];
    char fname[128];
    char *data;
    GtkTextMark *mark;
    static int currchip=-1;
    long int addr = 0;
    int id;
    int value;

    /* for asm code loading */
    static long int currline = 0;
    static struct func_data *oldfunc = NULL;

    if(strcmp(buff, "[atemu]('?' for help) \n") == 0){
        pipe_write_safe = 1;
    }else if(strncmp(buff, "[ATmega128L Emulator ", strlen("[ATmega128L Emulator ")) == 0){
        /* if it was able to scan in a version */
        char *version;
        
        version = malloc(strlen(buff) - strlen("[ATmega128L Emulator "));
        sscanf(buff, "[ATmega128L Emulator %[^]]]", version);

        if(strcmp(version, PACKAGE_VERSION) != 0){
            GtkWidget *dialog;

            dialog = gtk_message_dialog_new( GTK_WINDOW(window1),
                GTK_DIALOG_MODAL,
                GTK_MESSAGE_WARNING,
                GTK_BUTTONS_OK,
                "The emulator backend has version %s whereas xatdb has version %s.\n\nUnexpected behaviour may result.", version, PACKAGE_VERSION);
            gtk_dialog_run(GTK_DIALOG(dialog));
            gtk_widget_destroy(dialog);
        }

        free(version);
    } else if (strncmp(buff, "Loading Chip ", strlen("Loading Chip ") ) == 0 ) {
        /* the chipname can be a rather arbitrary string; This code attempts
         * to make no assumptions about it */
        char *start, *end;

        start = strchr(buff, '"');
        end = strrchr(buff, '"');

        /* grab the current chip num */
        sscanf(end, "\" (%d", &currchip );
        /* currchip is zero based */
        currchip = currchip - 1;
        
        /* pull out chip name */
        *start = '\0';
        *end = '\0';
        start++;
        chip_array[currchip].name = strdup( start );
        gtk_notebook_set_tab_label( GTK_NOTEBOOK( notebook_chips ), 
            gtk_notebook_get_nth_page( GTK_NOTEBOOK( notebook_chips ), currchip + 1 ),
            gtk_label_new( chip_array[currchip].name ) );
        gtk_widget_show_all( gtk_notebook_get_nth_page( GTK_NOTEBOOK( notebook_chips ), currchip + 1 ) );
    } else if (strncmp(buff, "\tFlash: ", strlen("\tFlash: ") ) == 0 ) {
	char *start;
        char *end;

        /* ':' + 2 (to move past ": ") */
	start = strchr( buff, ':' ) + 2;
        end = strchr( buff, '\n' );
        *end = '\0';

        printf( "Flash loaded[%d]: [%s]\n", currchip, start );
	chip_array[currchip].program = g_string_new( start );

        parse_elf( start, currchip );
    } else if (strncmp(buff, "Number of ", strlen("Number of ") ) == 0 ) {
        sscanf(buff, "Number of chips: %d", &num_chips );
        create_all_chips( num_chips );
    } else if(strncmp(buff, "Current chip ", strlen("Current chip ")) == 0){
        sscanf(buff, "Current chip is now %d\n", &currchip);
        currline = 0;
        oldfunc = NULL;
    } else if(strncmp(buff, "Call stack ", strlen("Call stack ")) == 0 ) {
        sscanf(buff, "Call stack %d begin:\n", &currchip );
        gtk_list_store_clear( chip_array[currchip].stack );
    } else if(strncmp(buff, "\taddr: ", strlen("\taddr: ")) == 0 ) {
        sscanf(buff, "\taddr: %lx\n", &addr );
        add_stack_item( currchip, addr );
    }else if(strncmp(buff, "Error: ", strlen("Error: "))==0 ||
             strncmp(buff, "Warning: ", strlen("Warning: "))==0 ||
             strstr(buff, "parser error")!=NULL){
        /* print errors to the console */
        mark = text_buffer_printf( textbuff_console, "%s", buff );
        gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(textview_console), mark, 0.0, FALSE, 0.0, 0.0);
    }else if(strncmp(buff, "A0x", 3)==0 && currchip > -1) {
        struct func_data *func;
        
        sscanf(buff, "A%lx: ", &addr );

        /* see if we are in a different function than before */
        func = address_to_func_data(currchip, addr);
        if(func != oldfunc){
            GtkTextIter start, end;
            /* print the function name, and make it pretty */
            if(func != NULL){
                mark = text_buffer_printf(chip_array[currchip].code, "\n%s\n", func->name);
            }else{
                mark = text_buffer_printf(chip_array[currchip].code, "\n<unknown>\n");
            }
            gtk_text_buffer_get_iter_at_mark(chip_array[currchip].code, &start, mark);
            gtk_text_buffer_get_iter_at_line(chip_array[currchip].code, &end, gtk_text_iter_get_line(&start) - 1);
            gtk_text_buffer_apply_tag_by_name(chip_array[currchip].code, "function", &start, &end);

            currline+=2;
            oldfunc = func;
        }
        
        add_addr_line_mapping( currchip, addr, currline );
        text_buffer_printf(chip_array[currchip].code, "%s", buff);
        currline++;
    }else{
        if ( sscanf( buff, "%d %lld %s %n", &chipid, &cycles, label, &left ) )
        {
            data = buff + left;

            if ( strcmp( "event", label ) == 0 ){
                mark = text_buffer_printf( chip_array[chipid].events, "%16lld: %s", cycles, data);
                gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(chip_array[chipid].textview_events), mark, 0.0, FALSE, 0.0, 0.0);
                
                mark = text_buffer_printf( textbuff_console, "%s", buff );
                gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(textview_console), mark, 0.0, FALSE, 0.0, 0.0);
            } else if ( strcmp( "debug", label ) == 0 ) {
                if(strncmp(data, "notify ", strlen("notify ")) == 0) {
                    /* don't print out notify points */
                    sscanf(data, "notify addr %lx id %d = %x, chip %d\n", &addr, &id, &value, &chipid );

                    update_global( chipid, id, addr, value );
                }else if(strncmp("watchpoint", data, 10) == 0){
                    struct tuple_global_addr *glob;
                    int addr;
                    char *mode;
                    
                    mark = text_buffer_printf( textbuff_console, "%s", buff );
                    gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(textview_console), mark, 0.0, FALSE, 0.0, 0.0);
                    
                    /* Parse data into an address and a mode */
                    strtok(data, " ");
                    addr = strtol(strtok(NULL, " "), NULL, 0);
                    mode = strtok(NULL, " ");
                    
                    /* Check what the event (mode) is, and assign it to mode, allowing for internationalization */
                    if( strncmp( mode, "set", 3 ) == 0)
                    {
                        mode = _("set");
                    }
                    else if( strncmp( mode, "cleared", 7 ) == 0 )
                    {
                        mode = _("cleared");
                    }
                    else
                    {
                        mode = _("hit");
                    }
                    
                    /* Check if a global exists at the specified address; if so, assign it to glob */

                    if( ( glob = addr_to_global_data( chipid, addr ) ) != NULL )
                    {
                        mark = text_buffer_printf( chip_array[chipid].debug, _("%16lld: watchpoint \"%s\" (0x%x) %s.\n"),
                            cycles, glob->name, addr, mode );
                    }
                    else
                    {
                        mark = text_buffer_printf( chip_array[chipid].debug, _("%16lld: unknown watchpoint (0x%x) %s.\n"),
                            cycles, addr, mode );
                    }
                    gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(chip_array[chipid].textview_debug), mark, 0.0, FALSE, 0.0, 0.0);
                    
                }else if(strncmp("breakpoint", data, 10) == 0){
                    /* only print asm and source breakpoints (not auto) */
                    long int test_addr;
                    sscanf( data + 10, "%lx", &test_addr );
                    if( is_breakpoint_at_addr( chipid, test_addr )){
                        mark = text_buffer_printf( chip_array[chipid].debug, "%16lld: %s", cycles, data);
                        gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(chip_array[chipid].textview_debug), mark, 0.0, FALSE, 0.0, 0.0);

                        mark = text_buffer_printf( textbuff_console, "%s", buff );
                        gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(textview_console), mark, 0.0, FALSE, 0.0, 0.0);
                    }
                }else{
                    /* always print out events that aren't handled */
                    mark = text_buffer_printf( chip_array[chipid].debug, "%16lld: %s", cycles, data);
                    gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(chip_array[chipid].textview_debug), mark, 0.0, FALSE, 0.0, 0.0);
                    
                    mark = text_buffer_printf( textbuff_console, "%s", buff );
                    gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(textview_console), mark, 0.0, FALSE, 0.0, 0.0);
                }  
            } else if ( strcmp( "data", label ) == 0 ) {
                int c;
                long data_addr;
                unsigned int dumpdata[16];
                uint8_t dumpdata_fixed[16];

                sscanf( data, "%lx: %x %x %x %x %x %x %x %x %x %x %x %x %x %x %x %x", &data_addr,
                    &dumpdata[0], &dumpdata[1], 
                    &dumpdata[2], &dumpdata[3], 
                    &dumpdata[4], &dumpdata[5], 
                    &dumpdata[6], &dumpdata[7], 
                    &dumpdata[8], &dumpdata[9], 
                    &dumpdata[10], &dumpdata[11], 
                    &dumpdata[12], &dumpdata[13], 
                    &dumpdata[14], &dumpdata[15] );

                for( c = 0; c < 16; c++ )
                    dumpdata_fixed[c] = (uint8_t) dumpdata[c];

                if ( ( data_addr == 0x0000 ) || ( data_addr == 0x0010 ) ||
                     ( data_addr == 0x0050 ) ) {
                    refresh_registers( chipid, data_addr, dumpdata_fixed );
                }

                new_locals( chipid, data_addr, dumpdata_fixed );
                
                /* Useful for debugging 
                 * mark = text_buffer_printf( textbuff_console, "%s", buff );
                 * gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(textview_console), mark, 0.0, FALSE, 0.0, 0.0);
                 */
            }else if (strcmp("opcode", label) == 0){
                /* need to highlight the line in the asm code view */
                /* also in the C view */
                int line;
                long addr;
                char *cfile=NULL;
                GtkTextIter curr_iter;

                /* look up asm line */
                sscanf(data, "A%lx", &addr);

                /* update register tab: PC */
                set_register( chipid, REGS_TAB_PC, addr, 6, 7 );

                /* don't want to enable toolbar buttons before load finishes */
                if(atemu_load_finished){
                    toolbar_sense_stop();
                }

                line = address_to_text_line(chipid, addr);

                if(line >= 0){
                    gtk_text_buffer_get_iter_at_line(chip_array[chipid].code, &curr_iter, line);
                    
                    /* mark current line */
                    gutter_draw_stock(GTK_TEXT_VIEW(chip_array[chipid].textview_code), GUTTER_COL_PC, line, XATDB_STOCK_CURR_LINE );

                    /* scroll to buffer */
                    gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(chip_array[chipid].textview_code), &curr_iter, 0.1, FALSE, 0.0, 0.0);
                }

                /* C line stuff */
                line = address_to_c_line(chipid, addr, &cfile);
                if(line >= 1){
                    GtkTextBuffer *buff;
                    GtkWidget *view;
                    int index;
                    GtkWidget *menu_image;

                    buff = cfile_to_textbuffer(chipid, cfile);
                    index = cfile_to_cfile_index(chipid, cfile);
                    view = cfile_index_to_textview(chipid, index);

                    if(buff != NULL){
                        /* clear stuff from prior "current chip" */

                        /* set the current graphic in cfile_select */
                        menu_image = gtk_image_new_from_stock( XATDB_STOCK_CURR_LINE, GTK_ICON_SIZE_MENU );
                        gtk_widget_show( menu_image );
                        gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( cfile_index_to_menu_item( chipid, index ) ), menu_image );

                        chip_array[chipid].cfile_current_menu_index = index;
                        
                        /* and mark current line */
                        gtk_text_buffer_get_iter_at_line(buff, &curr_iter, line);
                        /* check if user is in a source file or not */
                        if( chip_array[chipid].textview_source != chip_array[chipid].textview_code ) {
                            /* load buffer */
                            if( chip_array[chipid].textview_source != view ) {
                                gtk_option_menu_set_history(GTK_OPTION_MENU(chip_array[chipid].cfile_select), index);
                            }
                        }
                        gutter_draw_stock(GTK_TEXT_VIEW(view), GUTTER_COL_PC, line, XATDB_STOCK_CURR_LINE);

                        /* scroll to program counter */
                        gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(view), &curr_iter, 0.1, FALSE, 0.0, 0.0);

                    }
                }

                atemu_dump_regs( chipid );

                update_locals( chipid, addr );
                
                mark = text_buffer_printf( textbuff_console, "%s", buff );
                gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(textview_console), mark, 0.0, FALSE, 0.0, 0.0);
            }
        }
    }
}

GtkWidget* create_window1 ()
{
    GtkWidget *vbox1;
    GtkWidget *toolbar1;
    GtkWidget *tmp_toolbar_icon;
    GtkWidget *console_scroll;
    GtkWidget *label8;
    GtkWidget *menubar;
    GtkAccelGroup *accel_group;
    GtkItemFactory *menus;
    /* Menu bar definition */
    static GtkItemFactoryEntry menu_items[] = {
        { _("/_File"),          NULL,        NULL,               0, "<Branch>" },
        { _("/File/_Open..."),  _("<CTRL>O"),   menu_file_open_activate,  0, "<StockItem>", GTK_STOCK_OPEN },
        { _("/File/sep1"),      NULL,        NULL,               0, "<Separator>" },
        { _("/File/_Quit"),     _("<CTRL>Q"),   menu_file_quit_activate,  0, "<StockItem>", GTK_STOCK_QUIT },
        { _("/_Edit"),			 NULL,		 NULL,			     0, "<Branch>" },
        { _("/Edit/_Copy"),	 _("<CTRL>C"),   menu_edit_copy_activate,  0, "<StockItem>", GTK_STOCK_COPY },
        { _("/Edit/sep1"),      NULL,        NULL,               0, "<Separator>" },
        { _("/Edit/Select _All"), _("<CTRL>A"), on_edit_select_all_activate, 0, "<Item>" },
        { _("/Edit/Dese_lect All"), _("<SHIFT><CTRL>A"), on_edit_deselect_all_activate, 0, "<Item>" },
        { _("/_Debug"),		 NULL,		 NULL,				 0, "<Branch>" },
        { _("/Debug/_ASM Step"), _("<SHIFT><CTRL>Right"), button_step_clicked, 0, "<StockItem>", XATDB_STOCK_STEP },
        { _("/Debug/Source S_tep"), _("<CTRL>Right"), button_cstep_clicked, 0, "<StockItem>", XATDB_STOCK_SOURCE_STEP },
        { _("/Debug/_Run"),	 _("<CTRL>R"),	 button_go_clicked,  0, "<StockItem>", XATDB_STOCK_RUN },
        { _("/Debug/_Stop"),    _("Escape"),		 button_stop_clicked, 0, "<StockItem>", XATDB_STOCK_STOP },
        { _("/Debug/R_estart"),	 _("<SHIFT><CTRL>R"), button_restart_clicked, 0, "<StockItem>", XATDB_STOCK_RESTART },
        { _("/Debug/sep1"),     NULL,       NULL,               0,  "<Separator>" },
        //{ _("/Debug/Set _Breakpoint"), _("<CTRL>B"), on_set_break_curr_activate, 0, "<StockItem>", XATDB_STOCK_BREAK_ON },
        //{ _("/Debug/_Clear Breakpoint"), _("<SHIFT><CTRL>B"), on_clear_break_curr_activate, 0, "<StockItem>", XATDB_STOCK_BREAK_OFF },
        { _("/Debug/Add Brea_kpoint at..."), _("<CTRL><MOD1>B"), on_add_at_activate, XATDB_BREAK_POPUP,  "<Item>" },
        { _("/Debug/C_lear Breakpoints on Current Chip"), _("<SHIFT><CTRL><MOD1>B"), debug_clear_all_clicked, 0, "<Item>" },
        { _("/Debug/sep2"),     NULL,       NULL,               0,  "<Separator>" },
        { _("/Debug/Set _Watchpoint"), _("<CTRL>W"), on_set_watch_activate, 0, "<StockItem>", XATDB_STOCK_WATCH_ON },
        { _("/Debug/Clear Watc_hpoint"), _("<SHIFT><CTRL>W"), on_clear_watch_activate, 0, "<StockItem>", XATDB_STOCK_WATCH_OFF },
        { _("/Debug/Add Watchpo_int at..."), _("<CTRL><MOD1>W"), on_add_at_activate, XATDB_WATCH_POPUP,  "<Item>" },
        { _("/Debug/Clear Watch_points on Current Chip"), _("<SHIFT><CTRL><MOD1>W"), on_clear_all_watchpoints_activate, 0, "<Item>" },
        { _("/_Help"),			 NULL,		 NULL,				 0,  "<Branch>" },
        //{ _("/Help/_Contents"),  _("F1"),       menu_help_contents_activate, 0, "<Item>" },
        { _("/Help/_About"),    NULL,        menu_help_about_activate, 0, "<Item>" },
    };
    gint nmenu_items = sizeof( menu_items ) / sizeof( menu_items[0] );

    /* Initialize and configure the treeview cell renderers */
    /* The renderer for plain text */
    renderer = gtk_cell_renderer_text_new ();
    g_object_set( G_OBJECT( renderer ), "family", "courier",  NULL );
    g_object_ref( renderer );
    
    /* The renderer for numbers (right justified) */
    data_renderer = gtk_cell_renderer_text_new ();
    g_object_set( G_OBJECT( data_renderer ), "xalign", 1.0 , "family", "courier", NULL);
    g_object_ref( data_renderer );
    
    /* The renderer for icons/pixbufs */
    pix_renderer = gtk_cell_renderer_pixbuf_new ();
    g_object_ref( pix_renderer );

    accel_group = gtk_accel_group_new ();

    /* Menu creation and handle assignment */
    menus = gtk_item_factory_new( GTK_TYPE_MENU_BAR, "<main>", accel_group );
    gtk_item_factory_create_items( menus, nmenu_items, menu_items, NULL );
    menubar = gtk_item_factory_get_widget( menus, "<main>");
    
    menu_debug_step = gtk_item_factory_get_item( menus, "/Debug/ASM Step" );
    menu_debug_cstep = gtk_item_factory_get_item( menus, "/Debug/Source Step" );
    menu_debug_run = gtk_item_factory_get_item( menus, "/Debug/Run" );
    menu_debug_stop = gtk_item_factory_get_item( menus, "/Debug/Stop" );
    menu_debug_restart = gtk_item_factory_get_item( menus, "/Debug/Restart" );
    menu_debug_add_break_at = gtk_item_factory_get_item( menus, "/Debug/Add Breakpoint at..." );
    menu_debug_add_watch_at = gtk_item_factory_get_item( menus, "/Debug/Add Watchpoint at..." );
    menu_debug_clear_breaks = gtk_item_factory_get_item( menus, "/Debug/Clear Breakpoints on Current Chip" );
    menu_debug_set_watch = gtk_item_factory_get_item( menus, "/Debug/Set Watchpoint" );
    menu_debug_clear_watch = gtk_item_factory_get_item( menus, "/Debug/Clear Watchpoint" );
    menu_debug_clear_watches = gtk_item_factory_get_item( menus, "/Debug/Clear Watchpoints on Current Chip" );
    menu_edit_copy = gtk_item_factory_get_item( menus, "/Edit/Copy" );
    menu_edit_select_all = gtk_item_factory_get_item( menus, "/Edit/Select All" );
    menu_edit_deselect_all = gtk_item_factory_get_item( menus, "/Edit/Deselect All" );
    
    /* Define the main window */
    window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (window1), _("Visual AVR Debugger & Simulator"));
    gtk_widget_set_size_request( window1, 750, 550 );
    g_signal_connect (G_OBJECT (window1), "destroy", G_CALLBACK (menu_file_quit_activate), NULL);
    g_signal_connect (G_OBJECT (window1), "delete_event", G_CALLBACK (menu_file_quit_activate), NULL);
    g_signal_connect (G_OBJECT (window1), "configure_event", G_CALLBACK (on_window1_configure), NULL);
	
    
    vbox1 = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (window1), vbox1);

    gtk_box_pack_start (GTK_BOX (vbox1), menubar, FALSE, FALSE, 0);
    
    /* Create the toolbar */
    toolbar1 = gtk_toolbar_new ();
    gtk_box_pack_start (GTK_BOX (vbox1), toolbar1, FALSE, FALSE, 0);
    gtk_toolbar_set_style (GTK_TOOLBAR (toolbar1), GTK_TOOLBAR_BOTH);

    tooltips = gtk_tooltips_new();

    /* Define the ASM Step button, button_step */
    tmp_toolbar_icon = gtk_image_new_from_stock (XATDB_STOCK_STEP, gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1)));
    button_step = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
            GTK_TOOLBAR_CHILD_BUTTON,
            NULL,
            _("ASM Step"),
            NULL, NULL,
            tmp_toolbar_icon, NULL, NULL);
    gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar1)->children)->data))->label), TRUE);
    g_signal_connect( G_OBJECT( button_step ), "clicked", G_CALLBACK( button_step_clicked ), NULL );

    gtk_tooltips_set_tip( GTK_TOOLTIPS( tooltips ), button_step, _("Run one line of disassembly"),
            _("Run the next line of disassembly then halt.") );

    /* Define the Source step button, button_cstep */
    tmp_toolbar_icon = gtk_image_new_from_stock (XATDB_STOCK_SOURCE_STEP, gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1)));
    button_cstep = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
            GTK_TOOLBAR_CHILD_BUTTON,
            NULL,
            _("Source Step"),
            NULL, NULL,
            tmp_toolbar_icon, NULL, NULL);
    gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar1)->children)->data))->label), TRUE);
    g_signal_connect( G_OBJECT( button_cstep ), "clicked", G_CALLBACK( button_cstep_clicked ), NULL );

    gtk_tooltips_set_tip( GTK_TOOLTIPS( tooltips ), button_cstep, _("Run one line of source"),
            _("Run the next line of source then halt.") );

    /* Define the Run button, button_go */
    tmp_toolbar_icon = gtk_image_new_from_stock (XATDB_STOCK_RUN, gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1)));
    button_go = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
            GTK_TOOLBAR_CHILD_BUTTON,
            NULL,
            _("Run"),
            NULL, NULL,
            tmp_toolbar_icon, NULL, NULL);
    gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar1)->children)->data))->label), TRUE);
    g_signal_connect( G_OBJECT( button_go ), "clicked", G_CALLBACK( button_go_clicked ), NULL );

    /* The tooltip that should appear once a file has loaded is set in atemu.c */
    gtk_tooltips_set_tip( GTK_TOOLTIPS( tooltips ), button_go, _("Run the program"),
            _("Run the program starting at the current line.") );

    /* Define the Stop button, button_stop */
    tmp_toolbar_icon = gtk_image_new_from_stock (XATDB_STOCK_STOP, gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1)));
    button_stop = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
            GTK_TOOLBAR_CHILD_BUTTON,
            NULL,
            _("Stop"),
            NULL, NULL,
            tmp_toolbar_icon, NULL, NULL);
    gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar1)->children)->data))->label), TRUE);
    g_signal_connect( G_OBJECT( button_stop ), "clicked", G_CALLBACK( button_stop_clicked ), NULL );

    gtk_tooltips_set_tip( GTK_TOOLTIPS( tooltips ), button_stop, _("Stop execution"),
            _("Stop execution.") );

    /* Define the Restart button, button_restart */
    tmp_toolbar_icon = gtk_image_new_from_stock( XATDB_STOCK_RESTART, gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar1)));
    button_restart = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
            GTK_TOOLBAR_CHILD_BUTTON,
            NULL,
            _("Restart"),
            NULL, NULL,
            tmp_toolbar_icon, NULL, NULL);
    gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar1)->children)->data))->label), TRUE);
    g_signal_connect( G_OBJECT( button_restart ), "clicked", G_CALLBACK( button_restart_clicked ), NULL );

    /* The tooltip that should appear once a file has loaded is set in atemu.c */
    gtk_tooltips_set_tip( GTK_TOOLTIPS( tooltips ), button_restart, _("Restart the program"),
            _("Restart the program being debugged.") );

    /* Removing set breakpoint and watchpoint (for now at least) */

    notebook_chips = gtk_notebook_new ();
    gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook_chips), TRUE);
    g_signal_connect(G_OBJECT(notebook_chips), "switch-page", G_CALLBACK(notebook_chips_select), NULL);
    gtk_box_pack_start (GTK_BOX (vbox1), notebook_chips, TRUE, TRUE, 0);

    console_scroll = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( console_scroll ), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS );
    gtk_container_add (GTK_CONTAINER (notebook_chips), console_scroll);

    textview_console = gtk_text_view_new ();
    gtk_text_view_set_editable( GTK_TEXT_VIEW( textview_console ), FALSE );
    gtk_text_view_set_cursor_visible( GTK_TEXT_VIEW( textview_console ), FALSE );
    gtk_text_view_set_wrap_mode( GTK_TEXT_VIEW( textview_console ), GTK_WRAP_WORD );
    textbuff_console = gtk_text_view_get_buffer( GTK_TEXT_VIEW( textview_console ) );

    g_signal_connect(G_OBJECT(textview_console), "event-after", G_CALLBACK(text_selected), NULL);
        
    tagtable = gtk_text_buffer_get_tag_table(textbuff_console);

    /* create text style tags */
    gtk_text_buffer_create_tag(textbuff_console, "function", 
        "foreground", "dimgrey", 
        "style", PANGO_STYLE_ITALIC,
        NULL);
    gtk_text_buffer_create_tag(textbuff_console, "monospace", 
        "family", "courier", 
        NULL);

    gtk_container_add (GTK_CONTAINER (console_scroll), textview_console);
    
    /* add status_table to the status bar */
    status_table = gtk_table_new( 1, 4, TRUE );
    gtk_box_pack_end( GTK_BOX( vbox1 ), status_table, FALSE, FALSE, 2 );
    
    /* add status bar to status_table*/
    statusbar1 = gtk_statusbar_new();
    g_object_ref( statusbar1 );
    gtk_statusbar_set_has_resize_grip( GTK_STATUSBAR( statusbar1 ), FALSE );
    gtk_table_attach_defaults( GTK_TABLE( status_table ), statusbar1, 0, 4, 0, 1 );
    
    /* create the progress bar, and add it to the status bar */
    progressbar1 = gtk_progress_bar_new();
    g_object_ref( progressbar1 );
    /* ensure that the status bar is always (at least) as tall as the progress bar */
    gtk_widget_set_size_request( progressbar1, 0, -1 );
    gtk_box_pack_end( GTK_BOX( statusbar1 ), progressbar1, FALSE, FALSE, 0 );

    /* label the console tab */
    label8 = gtk_label_new (_("Console"));
    gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_chips), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook_chips), 0), label8);
    gtk_label_set_justify (GTK_LABEL (label8), GTK_JUSTIFY_LEFT);

    gtk_window_add_accel_group (GTK_WINDOW (window1), accel_group);

    /* initialize the sensitivity of menu and toolbar widgets */
    gtk_widget_set_sensitive( menu_edit_copy, FALSE );
    toolbar_sense_set(FALSE);

    return window1;
}

void set_sensitive_data( int setting ) {
    /* Set or unsets the sensitivity of data views of all chips,
    and removes previous current line markers */
    
    int chip;
        
    for( chip = 0; chip < chip_array_cnt; chip++ ) {
        /* Set the sensitivity of the data treeviews */
        gtk_widget_set_sensitive( chip_array[ chip ].treeview_regs, setting );
        gtk_widget_set_sensitive( chip_array[ chip ].treeview_stack, setting );
        gtk_widget_set_sensitive( chip_array[ chip ].treeview_locals, setting );
        gtk_widget_set_sensitive( chip_array[ chip ].treeview_globals, setting );
    }
}

/* removes the PC icon from the cfile menu and ASM and C view gutters */
void remove_pc_icon(){
    int chip;
        
    for( chip = 0; chip < chip_array_cnt; chip++ ) {
        /* remove current line indicator from disassem textviews */
        gutter_clear_column( GTK_TEXT_VIEW(chip_array[chip].textview_code), GUTTER_COL_PC );

        if( chip_array[chip].cfile_current_menu_index != -1 ){
            /* remove old menu image */
            gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( cfile_index_to_menu_item( chip, chip_array[chip].cfile_current_menu_index ) ), NULL );

            /* remove current line indicator from source textview */
            gutter_clear_column( GTK_TEXT_VIEW( cfile_index_to_textview( chip, chip_array[chip].cfile_current_menu_index) ), GUTTER_COL_PC );
        }
    }
}

int verb_printf( int is_error, FILE *stream, char *formatting, ... ) {
    /* prints to the command line based on user-chosen verbosity settings. See xatdb --help.
     * is_error     Whether or not the printout is an error message.
     * stream       The file stream to print to, ( i.e. stdout )
     * formatting   The printf formatting string
     * args         additional outputs as specified in info
     * returns:     either the legnth of the output string, or a negative code
                    no output occured. -3000 is returned if nothing was printed
                    due to verbosity settings */
                    
    va_list args;
    int ret = -3000;
    
    va_start( args, formatting );
    
    if( verbose_level + is_error > 1 ) {
        /* OK to print */
        
        ret = vfprintf( stream, formatting, args );
        fflush( stream );
    }
    
    va_end( args );
    
    return ret;
}
